import os

import polib
from django.conf import settings
from django.core.management.base import BaseCommand

from django_restful_translator.utils import fetch_translatable_fields


class Command(BaseCommand):
    help = 'Generate .po files from DB translations'

    def handle(self, *args, **options):
        for language_set in settings.LANGUAGES:
            language = language_set[0]
            po_path = os.path.join(settings.BASE_DIR, 'drt_locale', language, 'LC_MESSAGES')
            os.makedirs(po_path, exist_ok=True)
            po_file_path = os.path.join(po_path, 'django.po')

            if os.path.isfile(po_file_path):
                po = polib.pofile(po_file_path)
            else:
                po = polib.POFile()

            translations = fetch_translatable_fields(language)

            for trans in translations:
                self.write_to_po_file(po, trans)

            po.metadata = {
                'Content-Type': 'text/plain; charset=UTF-8',
                'Content-Transfer-Encoding': '8bit',
            }
            po.save(po_file_path)

    def write_to_po_file(self, po, trans):
        entry = polib.POEntry(
            msgid=getattr(trans.content_object, trans.field_name),
            msgstr=trans.field_value,
            tcomment=f"{trans.content_object._meta.model_name}__{trans.field_name}__{trans.object_id}"
        )
        if not trans.field_value:
            entry.flags.append('fuzzy')
        po.append(entry)



from django.conf import settings
from django.utils import translation
from rest_framework import serializers

from django_restful_translator.models import TranslatableModel
from django_restful_translator.utils import get_translation
from .fields import GetTextCharField


class TranslatableDBSerializer(serializers.ModelSerializer):
    class Meta:
        model = TranslatableModel
        fields = '__all__'

    def to_representation(self, instance):
        data = super().to_representation(instance)
        for field in instance.translatable_fields:
            data[field] = get_translation(instance, field)
        return data


class TranslatableDBDictSerializer(serializers.ModelSerializer):
    class Meta:
        model = TranslatableModel
        fields = '__all__'

    def to_representation(self, instance):
        data = super().to_representation(instance)
        for field in instance.translatable_fields:
            data[field] = {settings.LANGUAGE_CODE: getattr(instance, field)} | get_translation(instance, field,
                                                                                               as_dict=True)
        return data


class TranslatableGettextSerializer(serializers.ModelSerializer):
    class Meta:
        model = TranslatableModel
        fields = '__all__'

    def to_representation(self, instance):
        data = super().to_representation(instance)
        for field in instance.translatable_fields:
            # Using gettext to get translations
            value = getattr(instance, field)
            data[field] = GetTextCharField().to_representation(value)
        return data


class TranslatableGettextDictSerializer(serializers.ModelSerializer):
    class Meta:
        model = TranslatableModel
        fields = '__all__'

    def to_representation(self, instance):
        data = super().to_representation(instance)
        for field in instance.translatable_fields:
            translations = {}
            for lang_code, lang_name in settings.LANGUAGES:
                value = getattr(instance, field)
                if lang_code == settings.LANGUAGE_CODE:
                    translations[lang_code] = value
                with translation.override(lang_code):
                    translated_value = GetTextCharField().to_representation(value)
                    if value != translated_value:
                        translations[lang_code] = translated_value
            data[field] = translations
        return data



from django.core.management.base import BaseCommand
from django_restful_translator.translation_providers import TranslationProvider
from django_restful_translator.utils import fetch_translatable_fields
from django.conf import settings


class Command(BaseCommand):
    help = 'Translate model fields'
    provider_names = [cls.name for cls in TranslationProvider.__subclasses__()]

    def add_arguments(self, parser):
        parser.add_argument(
            '--language',
            type=str,
            help='Specify the language to which the fields should be translated'
        )
        parser.add_argument(
            '--provider',
            type=str,
            help='Specify the translation provider to use for the translation. Available providers: ' + ', '.join(
                self.provider_names)
        )
        parser.add_argument(
            '--all',
            action='store_true',
            help='Translate even existing translations'
        )

    def handle(self, *args, **options):
        language = options['language']
        provider_name = options['provider']
        translate_all = options['all']

        if language not in [lang[0] for lang in settings.LANGUAGES]:
            self.stdout.write(f'Unknown language: {language}')
            return

        if language == settings.LANGUAGE_CODE:
            self.stdout.write('Cannot translate to the same language')
            return

        available_providers = {cls.name: cls for cls in TranslationProvider.__subclasses__()}
        provider_class = available_providers.get(provider_name)
        if not provider_class:
            self.stdout.write(f'Unknown provider: {provider_name}')
            return

        provider = provider_class()
        translations = fetch_translatable_fields(language)

        for trans in translations:
            if len(trans.field_value) > 0 and not translate_all:
                continue
            text = getattr(trans.content_object, trans.field_name)
            translated_text = provider.translate_text(text, settings.LANGUAGE_CODE, language)

            trans.field_value = translated_text
            trans.save()

            self.stdout.write(
                f'Translated {trans.content_object._meta.model_name} field {trans.field_name} to {language}')



import os
import polib
from django.apps import apps
from django.conf import settings
from django.core.management.base import BaseCommand
from django.contrib.contenttypes.models import ContentType

from django_restful_translator.models import TranslatableModel, Translation


class Command(BaseCommand):
    help = 'Import .po files to DB translations'

    def handle(self, *args, **options):
        # Get all the models in the project
        models = apps.get_models()
        # Filter out models that are instances of TranslatableModel
        translatable_models = [model for model in models if issubclass(model, TranslatableModel)]

        # Iterate over each language
        for language_set in settings.LANGUAGES:
            language = language_set[0]

            # Skip default language
            if language == settings.LANGUAGE_CODE:
                continue

            # Open the .po file for this language
            po_file_path = os.path.join(settings.BASE_DIR, 'drt_locale', language, 'LC_MESSAGES', 'django.po')

            # Check the last modification time of the .po file
            if os.path.isfile(po_file_path):
                po_file_mod_time = os.path.getmtime(po_file_path)

                # Get the last update time of the Translation objects for the current language
                last_translation_update = Translation.objects.filter(language=language).latest(
                    'updated_at').updated_at.timestamp()

                # Only proceed if the .po file is newer than the last update in the database
                if po_file_mod_time <= last_translation_update:
                    print(f"Skipping {language} because the .po file is older than the last update in the database")
                    continue

            if not os.path.isfile(po_file_path):
                continue

            po = polib.pofile(po_file_path)
            # Iterate over each entry in the .po file
            for entry in po:
                model_name, field_name, object_id = entry.tcomment.split("__")
                model = next((m for m in translatable_models if m._meta.model_name == model_name), None)
                if not model:
                    continue
                field_value = entry.msgstr
                if field_value == "":
                    continue
                # Fetch or create the Translation object
                trans, created = Translation.objects.get_or_create(
                    content_type=ContentType.objects.get_for_model(model),
                    object_id=object_id,
                    field_name=field_name,
                    language=language,
                    defaults={"field_value": field_value}
                )
                # If the Translation object already existed, update it
                if not created:
                    trans.field_value = entry.msgstr
                    trans.save()



import os
from abc import ABC, abstractmethod

import boto3
from django.conf import settings
from google.cloud import translate_v2


class TranslationProvider(ABC):
    name = "Base Provider"

    @abstractmethod
    def translate_text(self, text: str, source_language: str, target_language: str) -> str:
        pass


class GoogleTranslateProvider(TranslationProvider):
    name = "google_v2"

    def __init__(self):
        self.client = translate_v2.Client()

    def translate_text(self, text, source_language, target_language):
        result = self.client.translate(text, source_language=source_language, target_language=target_language)
        return result['translatedText']


class AWSTranslateProvider(TranslationProvider):
    name = "aws"

    def __init__(self):
        if not (hasattr(settings, 'AWS_ACCESS_KEY_ID') and hasattr(settings, 'AWS_SECRET_ACCESS_KEY') and hasattr(
                settings, 'AWS_REGION_NAME')):
            raise ValueError("AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY and AWS_REGION_NAME must be set in settings")
        self.client = boto3.client('translate', region_name=settings.AWS_REGION_NAME,
                                   aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
                                   aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY)

    def translate_text(self, text, source_language, target_language):
        response = self.client.translate_text(Text=text, SourceLanguageCode=source_language,
                                              TargetLanguageCode=target_language)
        return response['TranslatedText']
